# Provided as is with no warranties or guarantees.  
# Use at your own risk
# Not intended for production use!
# As always, test, test, test and then test some more
# The script always generates a log in c:\windows\temp named ResetReport.txt
# Author: scomurr

#Accepts the SDK service you want to connect to an a command of Execute or Report
Param($SDKService,$Action)

# If not Execute, then assumes report.
if($Action.toupper() -ne 'EXECUTE'){$Action = 'REPORT'}

# Import the module and connect to the SCOM MS
Import-Module OperationsManager
New-SCOMManagementGroupConnection $SDKService

# creates and preps the report file
out-file c:\windows\temp\ResetReport.TXT -encoding ASCII
(Get-Date) | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
"Starting" | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append

# get a hold of all non group classes
$classes = get-scomclass | where{($_.Singleton -eq $false)}

# iterate through the classes
foreach($class in $classes){
	
	# gets all of the instances of the specified class where the instance is not healthy and it is initialized.  Also, the instance has to be available or the reset will fail anyway.
	$MonitoringObjects = Get-SCOMClassInstance -class $class | where{($_.healthstate -ne 'Success') -AND ($_.healthstate -ne 'Uninitialized') -AND ($_.IsAvailable -eq $true)} | sort DisplayName
	# Reporting...
	"#######################################" | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
	$Class.DisplayName | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
	$MonitoringObjects | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
	
	# for each of the instances of class we just retrieved...
	foreach($MonitoringObject in $MonitoringObjects) {
		# get the unit monitors associated with the unhealthy instance of the class we just retrieved...
		$Monitors = $null
		$Monitors = get-scommonitor -Instance $MonitoringObject -recurse | Where{($_.xmltag.tostring() -eq 'UnitMonitor') -AND ($_.target.id.tostring() -eq $class.id.tostring())}
		# if there are no unit monitors then the healthy is rolling up through an agg or dep monitor.  We can move to the next class.
		if($Monitors -ne $null) {
			# colMonitors is created because the GetMonitorStates method accepts a very specific type of collection.  The next few lines simply take the monitors and effectively
			# casts the collection to the right type and stores that in colMonitors
			$colMonitors = $null
			$colMonitors = new-object "System.Collections.Generic.List[Microsoft.EnterpriseManagement.Configuration.ManagementPackMonitor]"
			$Monitors | foreach{$colMonitors.add($_)}
			
			# Now, retrieve the healthstates for each of those monitors as they are associated with the specific instance of the class.  If unhealthy, store it in ToReset.
			$ToReset = $null
			$ToReset = $MonitoringObject.GetMonitoringStates($colMonitors) | where{($_.HealthState -ne 'Success') -AND ($_.HealthState -ne 'Uninitialized')}
			# As long as we have a unit monitor that is in an unhealthy state for this specific instance, let's reset it!
			if($ToReset -ne $null){
				# we need to create a collection of monitors that only contains the monitors we want to reset.  No sense submitting reset tasks for all of the monitors if 
				# most of them are healthy anyway.  
				
				# This part stores the guids for each of the monitors we want to rest in a string
				$strToReset = $null
				$ToReset | foreach{$strToReset = $strToReset + " " + $_.MonitorID}
				
				# Reporting...
				$ToReset | foreach{$_.Name | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append}
				
				# this one filters out the collection of monitors we retrieved earlier.  At this point, the Monitors contains both healthy, unhealthy and uninitialized
				# after the next line, only the unhealthy monitors will remain
				$Monitors = $Monitors | where{$strToReset.toupper().Contains($_.ID.tostring().toupper())}
				
				# Reporting
				$Monitors | foreach{$_.Name | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append}
				
				# If Execute was specified at the command line, go ahead and do the reset.  Log the rest in the Reset log as well.
				if($Action.toupper() -eq 'EXECUTE'){
					$MonitoringObject.ResetMonitoringState($Monitors)
					"Resetting: " + $MonitoringObject.FullName | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
					$Monitors | foreach{$_.Name | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append}
				}
			}
		}
	}
}
# Reporting...
"Finished" | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
(Get-Date) | out-file c:\windows\temp\ResetReport.TXT -encoding ASCII -append
		